@@ -1,22 +1,16 @@
 use strict; use warnings;
 package Inline::Module;
-our $VERSION = '0.24';
+our $VERSION = '0.28';
 our $API_VERSION = 'v2';
 
-use Config();
-use File::Path();
-use File::Find();
 use Carp 'croak';
-
-# use XXX;
+use Config();
+use File::Find();
+use File::Path();
+use File::Spec();
 
 my $inline_build_path = './blib/Inline';
 
-sub new {
-    my $class = shift;
-    return bless {@_}, $class;
-}
-
 use constant DEBUG_ON => $ENV{PERL_INLINE_MODULE_DEBUG} ? 1 : 0;
 sub DEBUG { if (DEBUG_ON) { print "DEBUG >>> ", sprintf(@_), "\n" }}
 
@@ -34,10 +28,14 @@ sub import {
     my ($stub_module, $program) = caller;
 
     if ($program eq 'Makefile.PL' && not -e 'INLINE.h') {
+        $class->check_inc_inc($program);
         no warnings 'once';
         *MY::postamble = \&postamble;
         return;
     }
+    elsif ($program eq 'Build.PL') {
+        $class->check_inc_inc($program);
+    }
 
     return unless @_;
     my $cmd = shift;
@@ -75,6 +73,21 @@ Make sure you have the latest version of Inline::Module installed, then run:
     }
 }
 
+sub check_inc_inc {
+    my ($class, $program) = @_;
+    my $first = $INC[0] or die;
+    if ($first !~ /^(\.[\/\\])?inc[\/\\]?$/) {
+        die <<"...";
+First element of \@INC should be 'inc'.
+It's '$first'.
+Add this line to the top of your '$program':
+
+    use lib 'inc';
+
+...
+    }
+}
+
 sub importer {
     my ($class, $stub_module) = @_;
     return sub {
@@ -98,80 +111,43 @@ sub importer {
 }
 
 #------------------------------------------------------------------------------
-# The postamble methods:
+# The postamble method:
 #------------------------------------------------------------------------------
 sub postamble {
     my ($makemaker, %args) = @_;
 
-    my $inline = $args{inline}
+    my $meta = $args{inline}
         or croak "'postamble' section requires 'inline' key in Makefile.PL";
     croak "postamble 'inline' section requires 'module' key in Makefile.PL"
-        unless $inline->{module};
+        unless $meta->{module};
 
-    my $self = $Inline::Module::Self = Inline::Module->new;
-    $self->default_args($inline, $makemaker);
+    my $class = __PACKAGE__;
+    $class->default_meta($meta);
 
-    my $code_modules = $self->{module};
-    my $stub_modules = $self->{stub};
-    my @included_modules = $self->included_modules;
+    my $code_modules = $meta->{module};
+    my $stub_modules = $meta->{stub};
+    my $included_modules = $class->included_modules($meta);
 
     my $section = <<"...";
 distdir : distdir_inline
 
 distdir_inline : create_distdir
-\t\$(NOECHO) \$(ABSPERLRUN) -MInline::Module=distdir -e 1 -- \$(DISTVNAME) @$stub_modules -- @included_modules
+\t\$(NOECHO) \$(ABSPERLRUN) -MInline::Module=distdir -e 1 -- \$(DISTVNAME) @$stub_modules -- @$included_modules
 
 pure_all ::
 ...
     for my $module (@$code_modules) {
         $section .=
-            "\t\$(NOECHO) \$(ABSPERLRUN) -Iinc -Ilib -e 'use $module'\n";
+            "\t\$(NOECHO) \$(ABSPERLRUN) -Iinc -Ilib -M$module -e 1 --\n";
     }
     $section .=
-        "\t\$(NOECHO) \$(ABSPERLRUN) -Iinc -MInline::Module=fixblib -e 1";
+        "\t\$(NOECHO) \$(ABSPERLRUN) -Iinc -MInline::Module=fixblib -e 1 --\n";
 
     return $section;
 }
 
-sub default_args {
-    my ($self, $args, $makemaker) = @_;
-    $args->{module} = $makemaker->{NAME} unless $args->{module};
-    $args->{module} = [ $args->{module} ] unless ref $args->{module};
-    $args->{stub} ||= [ map "${_}::Inline", @{$args->{module}} ];
-    $args->{stub} = [ $args->{stub} ] unless ref $args->{stub};
-    $args->{ilsm} ||= 'Inline::C';
-    $args->{ilsm} = [ $args->{ilsm} ] unless ref $args->{ilsm};
-    %$self = %$args;
-}
-
-sub included_modules {
-    my ($self) = @_;
-    my $ilsm = $self->{ilsm};
-    my @include = (
-        'Inline',
-        'Inline::denter',
-        'Inline::Module',
-        @$ilsm,
-    );
-    if (caller eq 'Module::Build::InlineModule') {
-        push @include, 'Module::Build::InlineModule';
-    }
-    if (grep /:C$/, @$ilsm) {
-        push @include,
-            'Inline::C::Parser::RegExp';
-    }
-    if (grep /:CPP$/, @$ilsm) {
-        push @include,
-            'Inline::C',
-            'Inline::CPP::Config',
-            'Inline::CPP::Parser::RecDescent',
-            'Parse::RecDescent';
-    }
-    return @include;
-}
-
 #------------------------------------------------------------------------------
-# Class methods.
+# The handle methods.
 #------------------------------------------------------------------------------
 sub handle_stub {
     my ($class, $stub_module, $api_version) = @_;
@@ -230,7 +206,7 @@ sub handle_autostub {
     }
 
     push @INC, sub {
-        my ($self, $module) = @_;
+        my ($this, $module) = @_;
         delete $ENV{PERL5OPT};
 
         $module =~ s!\.pm$!!;
@@ -245,34 +221,16 @@ sub handle_autostub {
 
 sub handle_distdir {
     my ($class, $distdir, @args) = @_;
-    my (@inlined_modules, @included_modules);
+    my $stub_modules = [];
+    my $included_modules = [];
 
     while (@args and ($_ = shift(@args)) ne '--') {
-        push @inlined_modules, $_;
+        push @$stub_modules, $_;
     }
     while (@args and ($_ = shift(@args)) ne '--') {
-        push @included_modules, $_;
+        push @$included_modules, $_;
     }
-
-    my @manifest; # files created under distdir
-    for my $module (@inlined_modules) {
-        my $code = $class->dyna_module($module);
-        $class->write_module("$distdir/lib", $module, $code);
-        $code = $class->proxy_module($module);
-        $class->write_module("$distdir/inc", $module, $code);
-        $module =~ s!::!/!g;
-        push @manifest, "lib/$module.pm", "inc/$module.pm";
-    }
-    for my $module (@included_modules) {
-        my $code = $class->read_local_module($module);
-        $class->write_module("$distdir/inc", $module, $code);
-        $module =~ s!::!/!g;
-        push @manifest, "inc/$module.pm";
-    }
-
-    $class->add_to_manifest($distdir, @manifest);
-
-    return @manifest; # return a list of the files added
+    $class->add_to_distdir($distdir, $stub_modules, $included_modules);
 }
 
 sub handle_fixblib {
@@ -296,6 +254,76 @@ sub handle_fixblib {
     }, $inline_build_path);
 }
 
+#------------------------------------------------------------------------------
+# Worker methods.
+#------------------------------------------------------------------------------
+sub default_meta {
+    my ($class, $meta) = @_;
+    defined $meta->{module}
+        or die "Meta 'module' not defined";
+    $meta->{module} = [ $meta->{module} ] unless ref $meta->{module};
+    $meta->{stub} ||= [ map "${_}::Inline", @{$meta->{module}} ];
+    $meta->{stub} = [ $meta->{stub} ] unless ref $meta->{stub};
+    $meta->{ilsm} ||= 'Inline::C';
+    $meta->{ilsm} = [ $meta->{ilsm} ] unless ref $meta->{ilsm};
+}
+
+sub included_modules {
+    my ($class, $meta) = @_;
+    my $ilsm = $meta->{ilsm};
+    my $include = [
+        'Inline',
+        'Inline::denter',
+        'Inline::Module',
+        @$ilsm,
+    ];
+    if (caller eq 'Module::Build::InlineModule') {
+        push @$include, 'Module::Build::InlineModule';
+    }
+    if (grep /:C$/, @$ilsm) {
+        push @$include,
+            'Inline::C::Parser::RegExp';
+    }
+    if (grep /:CPP$/, @$ilsm) {
+        push @$include, (
+            'Inline::C',
+            'Inline::CPP::Config',
+            'Inline::CPP::Parser::RecDescent',
+            'Parse::RecDescent',
+            'ExtUtils::CppGuess',
+            'Capture::Tiny',
+        );
+    }
+    return $include;
+}
+
+sub add_to_distdir {
+    my ($class, $distdir, $stub_modules, $included_modules) = @_;
+    my $manifest = []; # files created under distdir
+    for my $module (@$stub_modules) {
+        my $code = $class->dyna_module($module);
+        $class->write_module("$distdir/lib", $module, $code);
+        $code = $class->proxy_module($module);
+        $class->write_module("$distdir/inc", $module, $code);
+        $module =~ s!::!/!g;
+        push @$manifest, "lib/$module.pm"
+            unless -e "lib/$module.pm";
+        push @$manifest, "inc/$module.pm";
+    }
+    for my $module (@$included_modules) {
+        my $code = $module eq 'Inline::CPP::Config'
+        ? $class->read_share_cpp_config
+        : $class->read_local_module($module);
+        $class->write_module("$distdir/inc", $module, $code);
+        $module =~ s!::!/!g;
+        push @$manifest, "inc/$module.pm";
+    }
+
+    $class->add_to_manifest($distdir, @$manifest);
+
+    return $manifest; # return a list of the files added
+}
+
 sub read_local_module {
     my ($class, $module) = @_;
     eval "require $module; 1" or die $@;
@@ -309,6 +337,18 @@ sub read_local_module {
     return $code;
 }
 
+sub read_share_cpp_config {
+    my ($class) = @_;
+    require File::Share;
+    my $dir = File::Share::dist_dir('Inline-Module');
+    my $path = File::Spec->catfile($dir, 'CPPConfig.pm');
+    open IN, '<', $path
+        or die "Can't open '$path' for input:\n$!";
+    my $code = do {local $/; <IN>};
+    close IN;
+    return $code;
+}
+
 sub proxy_module {
     my ($class, $module) = @_;
 
@@ -355,6 +395,8 @@ bootstrap $module;
 sub write_module {
     my ($class, $dest, $module, $code) = @_;
 
+    $code =~ s/\n+__END__\n.*//s;
+
     my $filepath = $module;
     $filepath =~ s!::!/!g;
     $filepath = "$dest/$filepath.pm";
@@ -375,24 +417,46 @@ sub add_to_manifest {
     my ($class, $distdir, @files) = @_;
     my $manifest = "$distdir/MANIFEST";
 
-    # XXX Module::Build thing.
-    # XXX Call Module::Build->_add_to_manifest
-    my $chmod = 0;
-    if (not -w $manifest) {
-        chmod 0644, $manifest;
-        $chmod = 1;
-    }
-
-    open my $out, '>>', $manifest
-        or die "Can't open '$manifest' for append:\n$!";
-    for my $file (@files) {
-        print $out "$file\n";
-    }
-    close $out;
-
-    if ($chmod) {
-        chmod 0444, $manifest;
+    if (-w $manifest) {
+        open my $out, '>>', $manifest
+            or die "Can't open '$manifest' for append:\n$!";
+        for my $file (@files) {
+            print $out "$file\n";
+        }
+        close $out;
     }
 }
 
+sub smoke_system_info_dump {
+    my ($class, @msg) = @_;
+    my $msg = sprintf(@msg);
+    chomp $msg;
+    require Data::Dumper;
+    local $Data::Dumper::Sortkeys = 1;
+    local $Data::Dumper::Terse = 1;
+    local $Data::Dumper::Indent = 1;
+
+    my @path_files;
+    File::Find::find({
+        wanted => sub {
+            push @path_files, $File::Find::name if -f;
+        },
+    }, File::Spec->path());
+    my $dump = Data::Dumper::Dumper(
+        {
+            'ENV' => \%ENV,
+            'Config' => \%Config::Config,
+            'Path Files' => \@path_files,
+        },
+    );
+    Carp::confess <<"..."
+Error: $msg
+
+System Data:
+$dump
+
+Error: $msg
+...
+}
+
 1;
